/*
 * freq_synths.c
 *
 *  Created on: 14.09.2017
 *      Author: rieche
 */

#include "freq_synths.h"

#define WR_CMD			0x02
#define RD_CMD			0x03
#define RD_STAT_CMD		0x05
#define WR_EN_CMD		0x06

#define HI_BYTE(x)		(x >> 8)
#define LO_BYTE(x)		(x & 0xff)

static void select_eeprom(spi_instance_t * g_spi, uint8_t select_byte);
static void select_registers(spi_instance_t * g_spi);

extern void LED_4(int state);


void freq_synths_ctrl(
		UART_instance_t * g_uart,
		freq_synths_t   * fs
)
{
	int      done = 0;
	uint16_t address = 0;
	uint8_t  value = 0;
	uint8_t  rx_buff[1], tx_buff[4];

	UART_polled_tx_string(g_uart, "\n\rfreq_synths_ctrl:\n\r");

	SPI_set_slave_select(fs->g_spi, fs->spi_slave);
	select_registers(fs->g_spi);

	tx_buff[0] = WR_CMD;

	while(!done) {
		UART_polled_tx_string(g_uart, "16-bit-address: ");
		address = stoi(get_input(g_uart));
		UART_polled_tx_string(g_uart, "8-bit-value: ");
		value = stoi(get_input(g_uart));

		tx_buff[1] = HI_BYTE(address);
		tx_buff[2] = LO_BYTE(address);
		tx_buff[3] = value;
		SPI_transfer_block(fs->g_spi, tx_buff, 4, rx_buff, 0);

		UART_polled_tx_string(g_uart, "quit with q: ");
		if(0 == strcmp(get_input(g_uart),"q"))
			done = 1;
	}

	SPI_clear_slave_select(fs->g_spi, fs->spi_slave);
	UART_polled_tx_string(g_uart, "\n\rquit freq_synths_ctrl\n\r");

	return;
}

void freq_synths_eeprom_init(
		UART_instance_t * g_uart,
		freq_synths_t   * fs
)
{
	uint8_t rx_buff[16], tx_buff[35];
	int     i, j;

	UART_polled_tx_string(
			g_uart,
			"\n\rfreq_synths_eeprom_init:\n\r Bytes to write:\n\r"
	);
	UART_polled_tx_string(
				g_uart,
				itoa(fs->eeprom_length)
	);

	SPI_set_slave_select(fs->g_spi, fs->spi_slave);
	select_eeprom(fs->g_spi, fs->eeprom_select_byte);


	for(j = 0; j < fs->eeprom_length; j+= 32) {
		tx_buff[0] = WR_EN_CMD;
		SPI_transfer_block(fs->g_spi, tx_buff, 1, rx_buff, 0);

		tx_buff[0] = WR_CMD;
		tx_buff[1] = HI_BYTE(j);
		tx_buff[2] = LO_BYTE(j);

		for(i = 0; i < 32; i++)
			tx_buff[i + 3] = fs->eeprom_data[i + j];

		SPI_transfer_block(fs->g_spi, tx_buff, 35, rx_buff, 0);
		LED_4(1);

		tx_buff[0] = RD_STAT_CMD;
		rx_buff[0] = 0x01;
		while(rx_buff[0] & 0x01)
			SPI_transfer_block(fs->g_spi, tx_buff, 1, rx_buff, 1);

		LED_4(0);
	}
	UART_polled_tx_string(g_uart, "\n\r fertig ");
	SPI_clear_slave_select(fs->g_spi, fs->spi_slave);

	return;
}

void freq_synths_read_reg(
		UART_instance_t * g_uart,
		freq_synths_t   * fs,
		int               start_addr,
		int               end_addr,
		uint8_t         * s
)
{
	int     address;
	uint8_t rx_buff[1], tx_buff[3];

	UART_polled_tx_string(g_uart, "\n\r");
	UART_polled_tx_string(g_uart, s);
	UART_polled_tx_string(g_uart, "\n\r");

	SPI_set_slave_select(fs->g_spi, fs->spi_slave);
	select_registers(fs->g_spi);

	tx_buff[0] = RD_CMD;

	for(address = start_addr; address <= end_addr; address++) {
		tx_buff[1] = HI_BYTE(address);
		tx_buff[2] = LO_BYTE(address);
		SPI_transfer_block(fs->g_spi, tx_buff, 3, rx_buff, 1);
		UART_polled_tx_string(g_uart, "addr: ");
		UART_polled_tx_string(g_uart, htoa(address));
		UART_polled_tx_string(g_uart, ", value: ");
		UART_polled_tx_string(g_uart, btoa(rx_buff[0]));
		UART_polled_tx_string(g_uart, "\n\r");
	}

	UART_polled_tx_string(g_uart, "\n\r");
	SPI_clear_slave_select(fs->g_spi, fs->spi_slave);

	return;
}

void freq_synths_read_eeprom(
		UART_instance_t * g_uart,
		freq_synths_t   * fs
)
{
	uint8_t rx_buff[1], tx_buff[4];
	int     i, j, address;

	UART_polled_tx_string(g_uart, "\n\r freq_synths_read_eeprom\n\r");
	SPI_set_slave_select(fs->g_spi, fs->spi_slave);
	select_eeprom(fs->g_spi, fs->eeprom_select_byte);

	tx_buff[0] = RD_CMD;

	for(i = 0; i < 256; i += 8) {
		UART_polled_tx_string(g_uart, "addr: ");
		UART_polled_tx_string(g_uart, htoa(i));

		for(j = 0; j < 8; j++) {
			UART_polled_tx_string(g_uart, " ");
			address = (i + j);
			tx_buff[1] = HI_BYTE(address);
			tx_buff[2] = LO_BYTE(address);
			SPI_transfer_block(fs->g_spi, tx_buff, 3, rx_buff, 1);
			UART_polled_tx_string(g_uart, btoa(rx_buff[0]));
		}

		UART_polled_tx_string(g_uart, "\n\r");
	}
	SPI_clear_slave_select(fs->g_spi, fs->spi_slave);

	return;
}

static void select_eeprom(spi_instance_t * g_spi, uint8_t select_byte)
{
	uint8_t rx_buff[1], tx_buff[4];

	tx_buff[0] = WR_CMD;
	tx_buff[1] = 0x00;
	tx_buff[2] = 0x00;
	tx_buff[3] = select_byte;
	SPI_transfer_block(g_spi, tx_buff, 4, rx_buff, 0);

	return;
}

static void select_registers(spi_instance_t * g_spi)
{
	uint8_t rx_buff[1], tx_buff[4];

	tx_buff[0] = WR_CMD;
	tx_buff[1] = 0x00;
	tx_buff[2] = 0x00;
	tx_buff[3] = 0x00;
	SPI_transfer_block(g_spi, tx_buff, 4, rx_buff, 0);

	return;
}
