/*
 * uart_terminal.c
 *
 *  Created on: 14.09.2017
 *      Author: rieche
 */

#include "uart_terminal.h"

uint8_t * get_input(UART_instance_t * g_uart)
{
	static uint8_t buffer[81], key;
	uint32_t done = 0, i = 0;

	while(!done)
		if(1 == UART_get_rx(g_uart, &key, 1)) {
			if(i >= (sizeof(buffer) - 1)) {
				i = 0;
				UART_polled_tx_string(g_uart, "Input buffer overflow\n\r");
			} else {
				switch(key) {
				case '\r': {
					buffer[i] = '\0';
					done = 1;
					UART_polled_tx_string(g_uart, "\n\r");
					break;
				}
				case '\177': { /* back space, delete char... */
					if(i != 0)
						--i;
					break;
				}
				default: buffer[i++] = key; break;
				}
				UART_send(g_uart, &key, 1);
			}
		}
	return buffer;
}

int strcmp(uint8_t *s, uint8_t *t)
{
	while(*s++ == *t++)
		if(*s == '\0')
			return 0;
	return *s - *t;
}

uint8_t * itoa(uint32_t value)
{
	static uint8_t string[11];
	int i;

	for(i = 9; i >= 0; i--) {
		string[i] = '0';
		if(value > 0) {
			string[i] += (value % 10);
			value /= 10;
		}
	}

	string[10] = '\0';

	return string;
}

uint8_t * htoa(uint32_t value)
{
	static uint8_t string[9] = {'0'};
	int i;

	for(i = 7; i >= 0; i--) {
		string[i] = (0x0000000F & value);
		if(string[i] < 0xA)
			string[i] += '0';
		else
			string[i] += ('A' - 0xA);

		value >>= 4;
	}
	string[8] = '\0';

	return string;
}

uint8_t * btoa(uint8_t value)
{
	static uint8_t string[3] = {'0'};
	int i;

	for(i = 1; i >= 0; i--) {
		string[i] = (0x0F & value);
		if(string[i] < 0xA)
			string[i] += '0';
		else
			string[i] += ('A' - 0xA);

		value >>= 4;
	}
	string[2] = '\0';

	return string;
}


int atoi(uint8_t a)
{
	return (a - '0');
}

int stoi(uint8_t * string)
{
	int retval = 0;
	uint8_t digit;

	while(*string != '\0') {
		if(*string >= '0' && *string <= '9')
			digit = (*string - '0');
		else if(*string >= 'A' && *string <= 'F')
			digit = (*string - 'A' + 0xA);
		else if(*string >= 'a' && *string <= 'f')
			digit = (*string - 'a' + 0xA);

		retval = 16 * retval + digit;
		string++;
	}
	return retval;
}

